--- Modules/fcntlmodule.c.orig	2006-09-27 15:17:32.000000000 -0400
+++ Modules/fcntlmodule.c	2010-06-24 13:23:59.000000000 -0400
@@ -95,7 +95,15 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 {
 #define IOCTL_BUFSZ 1024
 	int fd;
+#if defined(__APPLE__) || defined(__DragonFlyBSD__) || defined(__FreeBSD__) || \
+    defined(__FreeBSD_kernel__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#define IOCTL_ULONG 1
+#endif
+#ifdef IOCTL_ULONG
+	unsigned long code;
+#else
 	int code;
+#endif
 	int arg;
 	int ret;
 	char *str;
@@ -103,7 +111,11 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	int mutate_arg = 0;
 	char buf[IOCTL_BUFSZ+1];  /* argument plus NUL byte */
 
+#ifdef IOCTL_ULONG
+	if (PyArg_ParseTuple(args, "O&kw#|i:ioctl",
+#else
 	if (PyArg_ParseTuple(args, "O&iw#|i:ioctl",
+#endif
                              conv_descriptor, &fd, &code, 
 			     &str, &len, &mutate_arg)) {
 		char *arg;
@@ -164,7 +176,11 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	}
 
 	PyErr_Clear();
+#ifdef IOCTL_ULONG
+	if (PyArg_ParseTuple(args, "O&ks#:ioctl",
+#else
 	if (PyArg_ParseTuple(args, "O&is#:ioctl",
+#endif
                              conv_descriptor, &fd, &code, &str, &len)) {
 		if (len > IOCTL_BUFSZ) {
 			PyErr_SetString(PyExc_ValueError,
@@ -186,7 +202,11 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	PyErr_Clear();
 	arg = 0;
 	if (!PyArg_ParseTuple(args,
+#ifdef IOCTL_ULONG
+	     "O&k|i;ioctl requires a file or file descriptor,"
+#else
 	     "O&i|i;ioctl requires a file or file descriptor,"
+#endif
 	     " an integer and optionally a integer or buffer argument",
 			      conv_descriptor, &fd, &code, &arg)) {
 	  return NULL;
@@ -204,6 +224,7 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	}
 	return PyInt_FromLong((long)ret);
 #undef IOCTL_BUFSZ
+#undef IOCTL_ULONG
 }
 
 PyDoc_STRVAR(ioctl_doc,
