--- Modules/fcntlmodule.c.orig	2009-05-24 11:46:13.000000000 -0400
+++ Modules/fcntlmodule.c	2010-06-24 13:20:31.000000000 -0400
@@ -34,7 +34,11 @@ fcntl_fcntl(PyObject *self, PyObject *ar
 {
 	int fd;
 	int code;
+#if defined(__linux__)
 	long arg;
+#else
+	int arg;
+#endif
 	int ret;
 	char *str;
 	Py_ssize_t len;
@@ -61,7 +65,11 @@ fcntl_fcntl(PyObject *self, PyObject *ar
 	PyErr_Clear();
 	arg = 0;
 	if (!PyArg_ParseTuple(args,
+#if defined(__linux__)
              "O&i|l;fcntl requires a file or file descriptor,"
+#else
+             "O&i|i;fcntl requires a file or file descriptor,"
+#endif
              " an integer and optionally a third integer or a string", 
 			      conv_descriptor, &fd, &code, &arg)) {
 	  return NULL;
@@ -97,20 +105,15 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 {
 #define IOCTL_BUFSZ 1024
 	int fd;
-	/* In PyArg_ParseTuple below, we use the unsigned non-checked 'I'
-	   format for the 'code' parameter because Python turns 0x8000000
-	   into either a large positive number (PyLong or PyInt on 64-bit
-	   platforms) or a negative number on others (32-bit PyInt)
-	   whereas the system expects it to be a 32bit bit field value
-	   regardless of it being passed as an int or unsigned long on
-	   various platforms.  See the termios.TIOCSWINSZ constant across
-	   platforms for an example of thise.
-
-	   If any of the 64bit platforms ever decide to use more than 32bits
-	   in their unsigned long ioctl codes this will break and need
-	   special casing based on the platform being built on.
-	 */
-	unsigned int code;
+#if defined(__APPLE__) || defined(__DragonFlyBSD__) || defined(__FreeBSD__) || \
+    defined(__FreeBSD_kernel__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#define IOCTL_ULONG 1
+#endif
+#ifdef IOCTL_ULONG
+	unsigned long code;
+#else
+	int code;
+#endif
 	int arg;
 	int ret;
 	Py_buffer pstr;
@@ -119,7 +122,11 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	int mutate_arg = 1;
  	char buf[IOCTL_BUFSZ+1];  /* argument plus NUL byte */
 
-	if (PyArg_ParseTuple(args, "O&Iw*|i:ioctl",
+#ifdef IOCTL_ULONG
+	if (PyArg_ParseTuple(args, "O&kw*|i:ioctl",
+#else
+	if (PyArg_ParseTuple(args, "O&iw*|i:ioctl",
+#endif
                              conv_descriptor, &fd, &code, 
 			     &pstr, &mutate_arg)) {
 		char *arg;
@@ -174,7 +181,11 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	}
 
 	PyErr_Clear();
-	if (PyArg_ParseTuple(args, "O&Is*:ioctl",
+#ifdef IOCTL_ULONG
+	if (PyArg_ParseTuple(args, "O&ks*:ioctl",
+#else
+	if (PyArg_ParseTuple(args, "O&is*:ioctl",
+#endif
                              conv_descriptor, &fd, &code, &pstr)) {
 		str = pstr.buf;
 		len = pstr.len;
@@ -201,7 +212,11 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	PyErr_Clear();
 	arg = 0;
 	if (!PyArg_ParseTuple(args,
-	     "O&I|i;ioctl requires a file or file descriptor,"
+#ifdef IOCTL_ULONG
+	     "O&k|i;ioctl requires a file or file descriptor,"
+#else
+	     "O&i|i;ioctl requires a file or file descriptor,"
+#endif
 	     " an integer and optionally an integer or buffer argument",
 			      conv_descriptor, &fd, &code, &arg)) {
 	  return NULL;
@@ -219,6 +234,7 @@ fcntl_ioctl(PyObject *self, PyObject *ar
 	}
 	return PyLong_FromLong((long)ret);
 #undef IOCTL_BUFSZ
+#undef IOCTL_ULONG
 }
 
 PyDoc_STRVAR(ioctl_doc,
